# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Instance'
        db.create_table('instances_instance', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label', self.gf('instances.fields.DNSLabelField')(unique=True, max_length=63, db_index=True)),
        ))
        db.send_create_signal('instances', ['Instance'])


    def backwards(self, orm):
        # Deleting model 'Instance'
        db.delete_table('instances_instance')


    models = {
        'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'})
        }
    }

    complete_apps = ['instances']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Instance.title'
        db.add_column('instances_instance', 'title',
                      self.gf('django.db.models.fields.CharField')(default='An instance', max_length=100),
                      keep_default=False)

        # Adding field 'Instance.description'
        db.add_column('instances_instance', 'description',
                      self.gf('django.db.models.fields.TextField')(default='', blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Instance.title'
        db.delete_column('instances_instance', 'title')

        # Deleting field 'Instance.description'
        db.delete_column('instances_instance', 'description')


    models = {
        'instances.instance': {
            'Meta': {'object_name': 'Instance'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('instances.fields.DNSLabelField', [], {'unique': 'True', 'max_length': '63', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['instances']


from django.test import TestCase, LiveServerTestCase
from django.test.client import Client
from django.test.utils import override_settings
from django.core.validators import ValidationError
from django.contrib.auth.models import User

from .models import Instance

FAKE_URL = 'testing.example.org:8000'

class InstanceClient(Client):
    def get(self, *args, **kwargs):
        kwargs['HTTP_HOST'] = FAKE_URL
        return super(InstanceClient, self).get(*args, **kwargs)

    def post(self, *args, **kwargs):
        kwargs['HTTP_HOST'] = FAKE_URL
        return super(InstanceClient, self).post(*args, **kwargs)

@override_settings(
    BASE_HOST='example.org',
    PASSWORD_HASHERS = ( 'django.contrib.auth.hashers.MD5PasswordHasher', ),
)
class InstanceTestCase(TestCase):
    client_class = InstanceClient

    def setUp(self):
        self.instance = Instance.objects.create(label='testing')
        user = User.objects.create_user(username='admin', email='admin@example.org', password='admin')
        user.instances.add(self.instance)
        self.client.login(username='admin', password='admin')

    def assertRedirects(self, *args, **kwargs):
        kwargs['host'] = FAKE_URL
        return super(InstanceTestCase, self).assertRedirects(*args, **kwargs)

@override_settings( SESSION_COOKIE_DOMAIN='127.0.0.1.xip.io' )
class InstanceLiveServerTestCase(LiveServerTestCase):
    def setUp(self):
        self.instance = Instance.objects.create(label='testing')
        user = User.objects.create_user(username='admin', email='admin@example.org', password='admin')
        user.instances.add(self.instance)

        self.selenium.get('%s%s' % (self.live_server_url, '/accounts/login/?next=/'))
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys('admin')
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys('admin')
        self.selenium.find_element_by_xpath('//input[@value="Log in"]').click()

# ---

class SimpleTest(TestCase):
    def test_instance_lower_casing(self):
        i = Instance(label='HELLO')
        self.assertEqual(i.label, 'hello')

    def test_bad_label(self):
        self.assertRaises(ValidationError, lambda: Instance(label='Spaces are not allowed'))
        self.assertRaises(ValidationError, lambda: Instance(label='Nor-a-symbol-such-as-^'))
        self.assertRaises(ValidationError, lambda: Instance(label="Nor-can-you-end-with--"))




from functools import wraps
from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator
from django.utils.decorators import available_attrs
from django.views.generic import UpdateView

from .models import Instance
from .forms import InstanceForm

def user_in_instance(view_func):
    """
    user_passes_test only passes request.user to the test function, but we need
    request itself in order to look at the instance. So wrap the wrapper, just
    so we don't need to duplicate e.g. the redirect-to-login code.
    """
    orig_wrapped_func = user_passes_test(lambda u: u.is_superuser)(view_func)
    @wraps(orig_wrapped_func, assigned=available_attrs(orig_wrapped_func))
    def _wrapped_view(request, *args, **kwargs):
        if request.is_user_instance:
            return view_func(request, *args, **kwargs)
        return orig_wrapped_func(request, *args, **kwargs)
    return _wrapped_view

class InstanceViewMixin(object):
    def get_queryset(self):
        return self.model.objects.for_instance(self.request.instance)

class InstanceFormMixin(InstanceViewMixin):
    @method_decorator(user_in_instance)
    def dispatch(self, *args, **kwargs):
        return super(InstanceFormMixin, self).dispatch(*args, **kwargs)

    def form_valid(self, form):
        # If not present, fill in the instance from the request
        # Clash of naming two things 'instance' here, sorry.
        if hasattr(form, 'instance') and not hasattr(form.instance, 'instance'):
            form.instance.instance = self.request.instance
        return super(InstanceFormMixin, self).form_valid(form)

class InstanceUpdate(InstanceFormMixin, UpdateView):
    model = Instance
    form_class = InstanceForm

    def get_object(self):
        return self.request.instance



import re

from django.db import models
from django.core.validators import RegexValidator
from django.utils.translation import ugettext as _

label_re = re.compile(r'(?i)^[a-z0-9][a-z0-9-]*[a-z0-9]$')
validate_label = RegexValidator(label_re, _("Enter a valid instance label consisting of letters, numbers, or hyphens."), 'invalid')

from south.modelsinspector import add_introspection_rules
add_introspection_rules([], ["^instances\.fields\.DNSLabelField"])

class DNSLabelField(models.CharField):
    description = "A DNS label"

    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 63)
        super(DNSLabelField, self).__init__(*args, **kwargs)
        self.validators.append(validate_label)

    def to_python(self, value):
        value = super(DNSLabelField, self).to_python(value)
        if value == '' or value is None:
            return value
        # Want immediate validation on any instance creation
        validate_label(value)
        return value.lower()




from django.db import models
from django.conf import settings
from django.contrib.auth.models import User

from .fields import DNSLabelField

class InstanceManager(models.Manager):
    def for_instance(self, instance):
        return self.get_query_set().filter(instance=instance)

class Instance(models.Model):
    label = DNSLabelField( db_index=True, unique=True )
    title = models.CharField( max_length=100 )
    description = models.TextField( blank=True )
    users = models.ManyToManyField(User, related_name='instances', blank=True)
    created_by = models.ForeignKey(User, related_name='created_instances', null=True, blank=True)

    def __unicode__(self):
        return u'Instance %s' % self.label

    def get_absolute_url(self):
        url = 'http://%s.%s' % (self.label, getattr(settings, 'BASE_HOST', '127.0.0.1.xip.io'))
        if getattr(settings, 'BASE_PORT', None):
            url += ':%s' % settings.BASE_PORT
        return url

class InstanceMixin(models.Model):
    instance = models.ForeignKey(Instance)

    objects = InstanceManager()

    class Meta:
        abstract = True




from django.conf.urls import patterns
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.views.generic import ListView

from instances.models import Instance

urlpatterns = patterns('',
    (r'^$', ListView.as_view(
        queryset = Instance.objects.all(),
        context_object_name = 'instances',
        template_name = 'instances/index.html',
    )),
)

urlpatterns += staticfiles_urlpatterns()

